/* Terraform - (C) 1997-2000 Robert Gasch (r.gasch@chello.nl)
 *  - http://212.187.12.197/RNG/terraform/
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>

#include "TFDialogConnect.h"
#include "GuiDialogOAC.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"



/*
 *  constructor: initialize all data members 
 */
TFDialogConnect::TFDialogConnect (HeightField *HF, HeightFieldDraw *HFD)
	        : TFPreviewDialog (HF, HFD, "Terraform Level Connect Dialog", 
				"Edge Connector Iterations", 6, 4),
		  d_hBoxLevelerIterations (TRUE, 0),
		  d_vBoxOptions (TRUE, 5),
		  d_adjLevelerIterations (5, 1, 50, 1),
		  d_hsLevelerIterations (d_adjLevelerIterations),
		  d_lblLevelerIterations (_("Number of Iterations")),
		  d_frmOptions (_("Leveler Options")) 

{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogConnect\n");

	buildDialogWindow ();
	this->setHFobjs (HF, HFD);
	sprintf (this->p_windowTitle, _("Level Connector: %s"), p_HF->getName());
	this->set_title (this->p_windowTitle);
	this->iterateEvents ();
	updatePreviewCallback ();

	d_adjLevelerIterations.value_changed.connect(SigC::slot(this,&TFDialogConnect::updatePreviewCallback));

}


/*
 *  destructor: clean up 
 */
TFDialogConnect::~TFDialogConnect ()
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogConnect\n");
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogConnect::insertOptions ()
{
	SanityCheck::bailout ((!this->p_frmBase), "TFDialogConnect called without Frame label",
		"TFDialogConnect::setHFobjs");

	d_frmOptions.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_frmOptions.set_border_width (5);

	d_hBoxLevelerIterations.pack_start (d_lblLevelerIterations, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblLevelerIterations.show ();
	d_hsLevelerIterations.set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsLevelerIterations.set_digits (0);
	d_hsLevelerIterations.set_draw_value (TRUE);
	d_hBoxLevelerIterations.pack_end (d_hsLevelerIterations, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsLevelerIterations.show ();
	d_vBoxOptions.pack_start (d_hBoxLevelerIterations, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hBoxLevelerIterations.show ();

	d_frmOptions.add (d_vBoxOptions);
	this->p_tblMain->attach (d_frmOptions, 3, 6, 1, 3);
	d_vBoxOptions.show ();
	d_frmOptions.show ();
	this->p_frmBase->show ();
	this->get_vbox()->show ();
}


/*
 *  buildDialogWindow:
 */
void TFDialogConnect::buildDialogWindow ()
{
	this->set_usize (450, 225);
	this->set_title (this->p_windowTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	this->insertPreview ();
	insertOptions();
	this->fillActionArea ();

	this->show ();
}


/*
 *  buttonCallbackApply: fold the height field 
 */
void TFDialogConnect::buttonCallbackApply ()
{
	float		value = d_adjLevelerIterations.get_value ();

	SanityCheck::bailout ((!p_HFO), "p_HFO==NULL", "TFDialogConnect::buttonCallbackApply");
	p_HFO->LevelConnector ((int)(value));
	p_HFD->draw ();

	// only do this if window stays open 
	if (b_applyHit)
		{
		this->setHFobjs (p_HF, p_HFD);
		updatePreviewCallback ();
		}

	this->b_applyHit = TRUE;
}


/*
 *  updatePreviewCallback: update the preview after a widget has been updated.
 */
void TFDialogConnect::updatePreviewCallback ()
{
	if (!d_cbUsePreview.get_active())
		return;

	float		value = d_adjLevelerIterations.get_value ();

	SanityCheck::bailout ((!p_HFPreview), "p_HFPreview==NULL", "TFDialogConnect");
        SanityCheck::bailout ((!p_HFOPreview), "p_HFOPreview==NULL", "TFDialogConnect");
        SanityCheck::bailout ((!p_HFDPreview), "p_HFDPreview==NULL", "TFDialogConnect");
	this->previewUpToDate ();
	p_HFPreview->restoreBackup ();
	p_HFOPreview->LevelConnector ((int)(value));
	p_HFDPreview->setColormap (p_HFD->getColormap());
	p_HFDPreview->draw ();
}

